﻿#include "toptemplateclass0a.h"
#include <QtWidgets>
#include <topcore/topcore.h>

/*!
 * \class TopTemplateClass0A
 * \inmodule templateclass
 * \brief 带背景的空模板，常用来做首页
 */

TopTemplateClass0A::TopTemplateClass0A(const QString &iModuleName,
                                       const QVariantMap &iUrlPars,
                                       QWidget *iParent)
    : TopClassAbs(iParent)
{
    initModule(iModuleName, iUrlPars);
    QString licenseKey = config("sys_license_key").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("sys_lang_list").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    initUi();
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
}

TopTemplateClass0A::~TopTemplateClass0A()
{
    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
}

/*!
 * \brief 初始化界面
 */
void TopTemplateClass0A::initUi()
{
    QWidget *centerWgt = new QWidget(this);
    setCentralWidget(centerWgt);
    QVBoxLayout *layout = new QVBoxLayout(centerWgt);
    layout->setMargin(0);
    layout->setSpacing(0);
    QLabel *label = new QLabel(this);
    QString backgroundImagePath = config("background_image_path").toString();
    backgroundImagePath = QDir(APP->appPlatformPath()).filePath(backgroundImagePath);
    QPixmap pixmap(backgroundImagePath);
#if 0
    // 下面的代码是之前的首页实现，暂时注释掉
    QPixmap temp(pixmap.size());
    temp.fill(Qt::transparent);
    QPainter painter(&temp);
    painter.setCompositionMode(QPainter::CompositionMode_Source);
    painter.drawPixmap(0, 0, pixmap);
    painter.setCompositionMode(QPainter::CompositionMode_DestinationIn);
    painter.fillRect(temp.rect(), QColor(0, 0, 0, 200));
    painter.end();
    pixmap = temp;
#endif
    label->setPixmap(pixmap);
    label->setScaledContents(true);
    label->setMinimumSize(100, 100);
    layout->addWidget(label);
}
