﻿#include "toptemplateclass4.h"
#include <QtWidgets>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tenumlist.h>
#include <tbaseutil/tfileio.h>
#include <tbaseutil/ttheme.h>
#include <tbaseutil/tresource.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <topcore/topcore.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topenummanager.h>
#include <toputil/t.h>
#include <twidget/tsplitter.h>
#include <twidget/ttableview.h>
#include <twidget/tcategorytreeview.h>
#include <twidget/tsearchentry.h>
#include <twidget/tpagetool.h>
#include <twidget/tuiloader.h>
#include <twidget/taccordion.h>
#include <twidget/tadvancedquery.h>
#include <twidget/tmessagebox.h>
#include <twidget/tscrollarea.h>
#include <twidget/ttabwidget.h>
#include <twidget/ttablechooserdialog.h>
#include <twidget/tuiloaderdialog.h>



TopTemplateClass4::TopTemplateClass4(const QString &iModuleName,
                                     const QVariantMap &iUrlPars,
                                     QWidget *iParent)
    : TopClassAbs(iParent)
{
    initModule(iModuleName, iUrlPars);
    QString licenseKey = config("sys_common").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("lang.conf").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    TopClassHelper::parseTableConf0(this, "view", &mTableConf);
    initUi();
    connect(this, SIGNAL(detailSaved(QString)),
            this, SLOT(onDetailSaved(QString)));
    connect(this, SIGNAL(isDetailModifiedChanged(bool)),
            this, SLOT(onDetailChanged(bool)));
    connect(this, SIGNAL(detailUidChanged(QString)),
            this, SLOT(refreshDetail(QString)));
    connect(this, SIGNAL(viewConfChanged()),
            this, SLOT(onViewConfChanged()));
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
    restoreSizeState();
    restoreObjectState(mBodySplitter);
    restoreObjectState(mTableView);

    refreshActionState();
    QTimer::singleShot(0, this, SLOT(refresh()));
    if (mNaviType == NaviType::CategoryTreeView) {
        (qobject_cast<TCategoryTreeView *>(mNaviView))->refresh();
    }

    auto refreshInterval = config("view.timing_refresh_interval", 0).toInt();
    if (refreshInterval > 0) {
        mRefreshTimer = new QTimer(this);
        connect(mRefreshTimer, &QTimer::timeout, [=]() {
            this->callAction("refresh");
        });
        mRefreshTimer->start(refreshInterval * 1000);
    }
}

TopTemplateClass4::~TopTemplateClass4()
{
    saveSizeState();
    saveObjectState(mBodySplitter);
    saveObjectState(mTableView);

    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
}

/*!
 * \brief 返回详细信息模块是否发生数据改变
 */
bool TopTemplateClass4::isDetailModified() const
{
    return mIsDetailModified;
}

/*!
 * \brief 刷新数据，\a iResetPage 表示是否要重置分页
 */
void TopTemplateClass4::refresh(bool iResetPage)
{
    TSqlSelectorV2 selector;
    if (!mTableConf.dbSql.isEmpty()) {
        selector.setTable(QString("(%1) _TEMP_TABLE_").arg(mTableConf.dbSql));
    } else if (!mTableConf.dbTableName.isEmpty()) {
        selector.setTable(mTableConf.dbTableName);
    } else {
        // db_sql为空和db_table_name为空时不作处理
        return;
    }
    selector.setField(mTableConf.queryFields);
    selector.setFieldFormat(mTableConf.fieldFormatMap);
    selector.setReturnRowCount(true);
    // 搜索分页
    TopClassHelper::handleSearchPageOnQuery(mSearchEntry, mPageTool, iResetPage, &selector);
    // 排序
    TopClassHelper::handleOrderOnQuery(mTableView, &mTableConf, &selector);
    // 只获取有效数据
    if (!mTableConf.dbDelFlagKey.isEmpty()) {
        selector.addWhere(QString("%1 = 0 OR %1 IS NULL").arg(mTableConf.dbDelFlagKey));
    }
    // 过滤项
    if(!mTableConf.dbFilter.isEmpty()) {
        selector.addWhere(mTableConf.dbFilter);
    }
    // 导航栏过滤项
    if (mNaviType == NaviType::None) {
        // do nothing
    } else if (mNaviType == NaviType::UiLoader) {
        QString whereStr = uiloaderWhere();
        if (!whereStr.isEmpty()) {
            selector.addWhere(whereStr);
        }
    } else if (mNaviType == NaviType::AdvancedQuery) {
        Q_ASSERT_X(false, "TopTemplateClass4", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::NaviFilter) {
        Q_ASSERT_X(false, "TopTemplateClass4", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::CategoryTreeViewAndAdvancedQuery) {
        if (!mCategoryWhere.isEmpty()) {
            selector.addWhere(mCategoryWhere.toSql());
        }
        TAdvancedQuery *advancedQuery = findAdvancedQuery();
        if (advancedQuery != nullptr) {
            if (!advancedQuery->getSql().isEmpty()) {
                selector.addWhere(advancedQuery->getSql());
            }
        }
    } else {
        if (!mCategoryWhere.isEmpty()) {
            selector.addWhere(mCategoryWhere.toSql());
        }
    }

    t::loading(this);
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
    unloading();
    TDataResponse response(data.toMap());
    if (response.hasError()) {
        alertError(ttr("Load data failed!"));
    } else {
        if (mPageTool != nullptr) {
            mPageTool->setRowCount(response.dataCount(), true);
        }
        QVariantList tableData = response.data().toList();
        TopClassHelper::formatTableData(this, &mTableConf, tableData);
        if (isHookExists("handleTableData")) {
            tableData = callHooksQuick("handleTableData", QVariantList() << QVariant(tableData)).toVariant().toList();
        }
        mTableView->loadData(tableData);
        mTableView->adjustViewSpan(mTableConf.mergeKeys);
        mSelectedItems.clear();
        setDetailUid(QString());
        alertOk(ttr("Data loaded"));
    }
}

/*!
 * \brief 刷新详细信息
 */
void TopTemplateClass4::refreshDetail(const QString &iUid)
{
    if (iUid.isEmpty()) {
        mDetailView->loadValues(QVariantMap(), true);
    } else {
        TSqlSelectorV2 selector;
        if (!mTableConf.dbSql.isEmpty()) {
            selector.setTable(QString("(%1) _TEMP_TABLE_").arg(mTableConf.dbSql));
        } else if (!mTableConf.dbTableName.isEmpty()) {
            selector.setTable(mTableConf.dbTableName);
        } else {
            //db_sql为空和db_table_name为空时不作处理
            return;
        }
        selector.setField(mTableConf.queryFields);
        selector.setWhere(mTableConf.primaryKey, iUid);
        selector.setFieldFormat(mTableConf.fieldFormatMap);
        TError err;
        QVariant data = runSqlQueryOnThreadSync(TOPSQLTHREAD_SELECT_MAP,
                                                QVariant::fromValue(selector),
                                                &err);
        if (err.isValid()) {
            alertError(ttr("Load data failed!"), err.text());
            return;
        }
        QVariantMap dataMap = data.toMap();
        mDetailView->loadValues(dataMap, true);
    }
    alertOk(ttr("Data loaded"));
    setIsDetailModified(false);
}

/*!
 * \brief 当前的详细信息的UID
 */
QString TopTemplateClass4::detailUid() const
{
    return mDetailUid;
}

/*!
 * \brief 设置详细信息的UID为 \a iUid,
 * 如果 \a iForceRefresh 为 true, 表示无论UID发生改变，都强制刷新
 */
void TopTemplateClass4::setDetailUid(const QString &iUid, bool iForceRefresh)
{
    if (iForceRefresh || mDetailUid != iUid) {
        mDetailUid = iUid;
        emit detailUidChanged(iUid);
    }
}

/*!
 * \brief 设置详细信息的数据改变状态
 */
void TopTemplateClass4::setIsDetailModified(bool iIsDetailModified)
{
    if (mIsDetailModified == iIsDetailModified) {
        return;
    }
    mIsDetailModified = iIsDetailModified;
    emit isDetailModifiedChanged(iIsDetailModified);
}

/*!
 * \brief 当前选中条目的primaryKey
 */
QVariantList TopTemplateClass4::selectedItems() const
{
    return mSelectedItems;
}

/*!
 * \brief 当前选中条目的所有数据
 */
QVariantList TopTemplateClass4::selectedDataMaps() const
{
    return mTableView->selectedRowDataMaps(true);
}

/*!
 * \brief 新建条目
 */
void TopTemplateClass4::newItem()
{
    mLastDetailUid = mDetailUid;
    setDetailUid(QString());
    setIsDetailModified(true);
}

/*!
 * \brief 保存条目
 */
void TopTemplateClass4::saveItem()
{
    QVariantList errLst = mDetailView->validateAll("COMMIT", true, "ERROR");
    if (!errLst.isEmpty()) {
        QStringList errStrLst;
        for (QVariant err : errLst) {
            errStrLst.append(err.toMap().value("text").toString());
        }
        alertError(ttr("Save data failed!"), errStrLst.join("\n"));
        return;
    }
    TSqlInserterV2 inserter;
    inserter.setTable(mTableConf.dbTableName);
    QVariantMap saveData;
    QMap<QString, QString> updatePolicy;
    TopClassHelper::fetchSaveFieldsFromUi(mDetailView, saveData, updatePolicy);
    inserter.setUniqueField(mTableConf.primaryKey);
    inserter.setAutoIncrementField(mTableConf.primaryKey);
    saveData.insert(mTableConf.primaryKey, mDetailUid);
    // 添加过滤项数据
    if (config("view.data_set.db_filter").type() == QVariant::Map) {
        QVariantMap dbFilterMap = config("view.data_set.db_filter").toMap();
        QMapIterator<QString, QVariant> i(dbFilterMap);
        while (i.hasNext()) {
            i.next();
            saveData.insert(i.key(), i.value());
        }
    }
    inserter.setData(saveData);
    QStringList fields = saveData.keys();
    fields.removeOne(mTableConf.primaryKey);
    inserter.setField(fields);
    // 设置更新策略
    {
        QMapIterator<QString, QString> i(updatePolicy);
        while (i.hasNext()) {
            i.next();
            inserter.setUpdatePolicy(i.key(), i.value());
        }
    }
    TError err;
    QVariant result = runSqlQueryOnThreadSync(TOPSQLTHREAD_REPLACE_ROW,
                                              QVariant::fromValue(inserter),
                                              &err);
    if (err.isValid()) {
        alertError(ttr("Save data failed!"), err.text());
        return;
    }
    emit detailSaved(result.toString());
    setIsDetailModified(false);
    alertOk(ttr("Data saved"));
}

/*!
 * \brief 重新载入条目
 */
void TopTemplateClass4::reloadItem()
{
    setDetailUid(mLastDetailUid.isEmpty() ? mDetailUid : mLastDetailUid, true);
}

/*!
 * \brief 复制条目
 */
void TopTemplateClass4::copyItem()
{
    mLastDetailUid = mDetailUid;
    mDetailUid = "";
    setIsDetailModified(true);
}

/*!
 * \brief 删除条目
 */
void TopTemplateClass4::deleteItems(const QVariantList &iItems)
{
    if (mTableConf.dbDelFlagKey.isEmpty()) {
        // 物理删除
        TSqlDeleterV2 deleter;
        deleter.setTable(mTableConf.dbTableName);
        deleter.setWhere(mTableConf.primaryKey, iItems);
        TError err;
        runSqlQueryOnThreadSync(TOPSQLTHREAD_DELETE_ROW,
                                QVariant::fromValue(deleter),
                                &err);
        if (err.isValid()) {
            alertError(ttr("Delete data failed!"), err.text());
        } else {
            refresh(true);
            alertOk(ttr("Data deleted"));
        }
    } else {
        // 逻辑删除
        TSqlUpdaterV2 updater;
        updater.setTable(mTableConf.dbTableName);
        updater.setWhere(mTableConf.primaryKey, iItems);
        updater.setData(QVariantMap {{mTableConf.dbDelFlagKey, 1}});
        TError err;
        runSqlQueryOnThreadSync(TOPSQLTHREAD_UPDATE_ROW,
                                QVariant::fromValue(updater),
                                &err);
        if (err.isValid()) {
            alertError(ttr("Delete data failed!"), err.text());
        } else {
            refresh(true);
            alertOk(ttr("Moved to recycle bin"));
        }
    }
}

/*!
 * \brief 表格控件对象
 */
TTableView *TopTemplateClass4::tableView() const
{
    return mTableView;
}

/*!
 * \brief 详细信息的表单对象
 */
TUiLoader *TopTemplateClass4::uiLoader() const
{
    return mDetailView;
}

/*!
 * \brief 导航控件
 */
QWidget *TopTemplateClass4::naviView() const
{
    return mNaviView;
}

/*!
 * \brief 当选中发生改变时的槽函数
 */
void TopTemplateClass4::onSelectionChanged()
{
    mSelectedItems = mTableView->selectedPrimaryKeys();
    QString curId;
    if (!mSelectedItems.isEmpty()) {
        curId = mSelectedItems.first().toString();
    }
    if (mDetailUid != curId) {
        mLastDetailUid = curId;
        if (!mIsDetailModified) {
            setDetailUid(curId);
        }
    }
    refreshActionState();
}

/*!
 * \brief 当分页发生改变时的槽函数
 */
void TopTemplateClass4::onPageChanged()
{
    refresh(false);
}

/*!
 * \brief 当导航栏过滤选项发生改变时的槽函数
 */
void TopTemplateClass4::onCategoryViewDataChanged(const QVariantList iDataLst)
{
    mCategoryWhere.clear();
    QString format = config("navi.format", "filter_by_enum").toString();
    if (!iDataLst.isEmpty()) {
        if (format.compare("filter_by_enum") == 0) {
            QVariantMap cateMap;
            for (QVariant item: iDataLst) {
                QVariantMap itemMap = item.toMap();
                QString data = itemMap.value("data").toString();
                QString name = itemMap.value("name").toString();
                if (data.isEmpty()) {
                    continue;
                }
                cateMap.insertMulti(data,name);
            }
            for (QString field: cateMap.uniqueKeys()) {
                QString dbField = field;
                TSqlWhereCompsiteV2 childWhere;
                childWhere.setLogic(TSqlWhereCompsiteV2::Logic_Or);
                for (QVariant ele: cateMap.values(field)) {
                    QString value = ele.toString();
                    if (mTableConf.fieldFormatMap.contains(dbField)) {
                        QString fieldValue = mTableConf.fieldFormatMap.value(dbField).toString();
                        if (fieldValue.compare("array") == 0) {
                            QString sqlStr = QString("%1 @> '{%2}' ").arg(dbField).arg(value);
                            childWhere.append(sqlStr);
                        }
                    } else {
                        childWhere.append(dbField, value);
                    }
                }
                mCategoryWhere.append(childWhere);
            }
        } else if (format.compare("filter_by_sql_where") == 0) {
            QVariantMap item = iDataLst.at(0).toMap();
            QString sqlWhere = item.value("data").toString();
            if (!sqlWhere.isEmpty()) {
                mCategoryWhere.append(sqlWhere);
            }
        }
    }
    refresh();
}

/*!
 * \brief 当详细信息表单数据发生改变时的槽函数
 */
void TopTemplateClass4::onDetailDataChanged()
{
    setIsDetailModified(true);
}

/*!
 * \brief 当详细信息表单保存成功时的槽函数
 */
void TopTemplateClass4::onDetailSaved(const QString &iUid)
{
    refresh(false);
    mTableView->selectRow(iUid);
    mDetailUid = iUid;
}

/*!
 * \brief 当详细信息表单数据改变状态发生变化时的槽函数
 */
void TopTemplateClass4::onDetailChanged(bool iIsDetailModified)
{
    mTableView->setEnabled(!iIsDetailModified);
    if (iIsDetailModified) {
        showMaskFrame(mMgtSplitter);
    } else {
        hideMaskFrame();
    }
    refreshActionState();
}

/*!
 * \brief 当module中view配置改变时的槽函数
 */
void TopTemplateClass4::onViewConfChanged()
{
    TopClassHelper::parseTableConf0(this, "view", &mTableConf);
}

/**
* \brief 当高级查询界面的添加按钮触发时的槽函数
*/
void TopTemplateClass4::onAddActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    TTableChooserDialog *dialog = new TTableChooserDialog(this);
    dialog->setTitle(tr("Please select field:"));
    dialog->setButtons(QStringList()
                       << ttr("Ok") + ":Ok:Yes"
                       << ttr("Cancel") + ":Cancel:Cancel"
                       << QString("%1:UnselectAll:ResetRole").arg(tr("Clear selection")));
    QVariantList headeritems;
    headeritems << QVariant() << TDataParse::stringList2VariantMap(
                       QStringList() << "name" << "title" << "display" << tr("Field") << "displayRole" << "$title"<<"resizeMode" << "Stretch");
    dialog->setHeaderItem(headeritems);
    dialog->setDataKeyList(QStringList() << "title" << "name");
    dialog->setPrimaryKey("name");
    dialog->setSearchKeys(QStringList() << "title");
    dialog->setLeftTableSelectionMode(QAbstractItemView::ExtendedSelection);
    dialog->setRightTableSelectionMode(QAbstractItemView::ExtendedSelection);
    dialog->tableView()->verticalHeader()->setVisible(false);
    dialog->selectedTableView()->verticalHeader()->setVisible(false);
    dialog->loadAllData(getAdvancedQueryFieldList());
    dialog->loadSelectedData(advancedQuery->getQueryData().value("value").toList());

    QVariantList ret = dialog->run();
    if (ret.isEmpty()) {
        return;
    }

    QStringList conditionLst;
    for (QVariant var : ret) {
        QString fieldName = var.toMap().value("name").toString();
        conditionLst << fieldName;
    }
    advancedQuery->clearSearchCondition();
    advancedQuery->addSearchCondition(conditionLst);
}

/**
* \brief 当高级查询界面的清除按钮触发时的槽函数
*/
void TopTemplateClass4::onClearActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }
    advancedQuery->clearSearchCondition();
}

/**
* \brief 当高级查询界面的另存为按钮触发时的槽函数
*/
void TopTemplateClass4::onSaveAsActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    QString fileName = QFileDialog::getSaveFileName(this,
                                                    tr("Save query criteria as file"),
                                                    QString(),
                                                    QString("*.cond"));
    if (fileName.isEmpty()) {
        return;
    }

    QVariantMap newConditionMap;
    newConditionMap.insert("condition", advancedQuery->getQueryData());
    QString conditionStr = TDataParse::variant2JsonStr(newConditionMap);
    TFileIo::writeFileString(conditionStr , fileName);
}

/**
* \brief 当高级查询界面的打开按钮触发时的槽函数
*/
void TopTemplateClass4::onOpenActtionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    QString fileName = QFileDialog::getOpenFileName(this,
                                                    tr("Open query criteria from file"),
                                                    ".",
                                                    tr("Condition file(*.cond)"));
    if (fileName.isEmpty()) {
        return;
    }
    QString conditionStr = TFileIo::readFileString(fileName);
    QVariantMap dataMap = TDataParse::jsonStr2Variant(conditionStr).toMap();
    QVariantMap conditionMap = dataMap.value("condition").toMap();
    advancedQuery->setData(conditionMap);
}

/**
* \brief 当高级查询界面的保存按钮触发时的槽函数
*/
void TopTemplateClass4::onSaveAsCommonQueryActionTriggered()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    TTableView *commonQueryTable = findCommonQueryTable();
    if (commonQueryTable == nullptr) {
        return;
    }

    QString uiStr;
    uiStr = "function func(self) {"
                   " return [{"
                           " name: 'basic_formlayout',"
                            "type: 'FormLayout',"
                            "property:{"
                                    "label_alignment: 'Top | Right',"
                                    "horizontal_spacing: 20, vertical_spacing: 10, margin: 20"
                            "},"
                            "child: [{"
                                    "name: 'name',"
                                    "type: 'LineEdit',"
                                    "title: '" + tr("Query Name") + "',"
                                    "pack: {"
                                            "label: '" + tr("Query Name") + "'"
                                    "},"
                                    "validate: 'NOTNULL'"
                                    "}]"
                            "}]}";

    TUiloaderDialog *dialog = new TUiloaderDialog(this);
    dialog->setTitle(tr("Save as a common query"));
    dialog->setSelf(this);
    dialog->resize(250,100);
    dialog->setButtons(QStringList() << QString("%1:Ok:AcceptRole").arg(ttr("Ok")) << QString("%1:Cancel::RejectRole").arg(ttr("Cancel")));
    dialog->setUiStr(uiStr);

    QVariantMap ret = dialog->run(true);
    if (ret.isEmpty()) {
        return;
    }

    // 判断name是否已经存在
    QString conditionName = ret.value("name").toString();
    QVariantList conditionList = mAdvancedQueryTableDataMap.value("advance").toList();
    for (QVariant var: conditionList) {
        QVariantMap itemMap = var.toMap();
        if (conditionName == itemMap.value("title")) {
            TMessageBox::error(this, tr("Save as common condition"), tr("This name already exists!"));
            return;
        }
    }

    // 取新的查询条件
    QVariantMap conditionMap = advancedQuery->getQueryData();
    QVariantMap newConditionMap;
    newConditionMap.insert("title", conditionName);
    newConditionMap.insert("condition", conditionMap);
    conditionList << QVariant(newConditionMap);

    commonQueryTable->loadData(conditionList);
    mAdvancedQueryTableDataMap.insert("advance", conditionList);

    // 生成json格式数据
    QString newConditionStr = TDataParse::variant2JsonStr(mAdvancedQueryTableDataMap);
    TSqlInserterV2 inserter;
    inserter.setTable("sys_conf");
    QVariantMap saveMap;
    saveMap.insert("data", newConditionStr);
    saveMap.insert("path", QStringList() << this->moduleName() << "AdvancedQuery");
    inserter.setData(saveMap);
    inserter.setField(saveMap.uniqueKeys());
    inserter.setAutoIncrementField("id");
    inserter.uniqueFieldRef() << "path";

    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_REPLACE_ROW, QVariant::fromValue(inserter));
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Load data failed!"), dataRes.errText());
        return;
    }
}

/**
* \brief 当常用查询界面的表格双击条目时的槽函数
*/
void TopTemplateClass4::onCommonQueryTableDoubleClicked(const QModelIndex &index)
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return;
    }

    TTableView *commonQueryTable = findCommonQueryTable();
    if (commonQueryTable == nullptr) {
        return;
    }

    TTabWidget *advancedQueryTabWidget = findAdvancedQueryTabWidget();
    if (advancedQueryTabWidget == nullptr) {
        return;
    }

    advancedQuery->clearSearchCondition();
    TTableModel *tableModel = qobject_cast<TTableModel *>(commonQueryTable->model());
    QVariantMap rowDataMap = tableModel->rowDataMap(index);
    QVariantMap conditionMap = rowDataMap.value("condition").toMap();
    advancedQuery->setData(conditionMap);
    advancedQueryTabWidget->setCurrentIndex(0);
}

/**
* \brief 当常用查询界面的表格删除条目时的槽函数
*/
void TopTemplateClass4::onDeleleCommonQueryActtionTriggered()
{
    TTableView *commonQueryTable = findCommonQueryTable();
    if (commonQueryTable == nullptr) {
        return;
    }

    QVariantMap selectedConditionMap = commonQueryTable->currentRowDataMap().toMap();
    if (selectedConditionMap.isEmpty()) {
        return;
    }

    QString selectedTitle = selectedConditionMap.value("title").toString();
    QVariantList conditionList = mAdvancedQueryTableDataMap.value("advance").toList();;
    QVariantList conditionDataList;
    for (QVariant var : conditionList){
        QString title = var.toMap().value("title").toString();
        // 如果不是选中的项，表示需要保留
        if (title != selectedTitle) {
            conditionDataList << QVariant(var.toMap());
        }
    }
    commonQueryTable->loadData(conditionDataList);
    mAdvancedQueryTableDataMap.insert("advance", conditionDataList);

    //生成json格式数据
    QString newConditionStr = TDataParse::variant2JsonStr(mAdvancedQueryTableDataMap);
    TSqlInserterV2 inserter;
    inserter.setTable("sys_conf");
    QVariantMap saveMap;
    saveMap.insert("data", newConditionStr);
    saveMap.insert("path", QStringList() << this->moduleName() << "AdvancedQuery");
    inserter.setData(saveMap);
    inserter.setField(saveMap.uniqueKeys());
    inserter.setAutoIncrementField("id");
    inserter.uniqueFieldRef() << "path";
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_REPLACE_ROW, QVariant::fromValue(inserter));
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Load data failed!"), dataRes.errText());
        return;
    }
}

/*!
 * \brief 初始化界面
 */
void TopTemplateClass4::initUi()
{
    mBodySplitter = new TSplitter(this);
    mBodySplitter->setObjectName("mBodySplitter");
    setCentralWidget(mBodySplitter);
    mMgtSplitter = new TSplitter(this);
    mBodySplitter->addWidget(mMgtSplitter);

    mBodyWgt = new QWidget(this);
    QVBoxLayout *bodyLayout = new QVBoxLayout(mBodyWgt);
    bodyLayout->setMargin(0);
    bodyLayout->setSpacing(0);
    mMgtSplitter->addWidget(mBodyWgt);
    initNaviView();

    mTableView = new TTableView(this);
    // 此ObjectName在其他逻辑中是有意义的，请勿删除
    mTableView->setObjectName("__view__");
    mTableView->setHeaderPopupEnabled(true);
    connect(mTableView->horizontalHeader(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)),
            this, SLOT(refresh()));
    connect(mTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onSelectionChanged()));
    connect(mTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SIGNAL(selectionChanged()));
    mTableView->setDataKeyList(mTableConf.dataKeys);
    mTableView->setPrimaryKey(mTableConf.primaryKey);
    QVariantList headerItems = mTableConf.horizontalHeaders;
    headerItems.prepend(QVariant());
    mTableView->setHeaderItem(headerItems);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MAIN_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
        mSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("MAIN_TOOLBAR/SEARCH_ENTRY"));
        if (mSearchEntry != nullptr) {
            connect(mSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(refresh()));
            mSearchEntry->setOptionList(TDataParse::headerItem2searchList(mTableView->headerItem()));
            mSearchEntry->setPlaceholderText(ttr("Search %1"));
        }
    }
    if (QMenu *tablePopup = qobject_cast<QMenu *>(uim()->getWidget("TABLEVIEW_POPUP"))) {
        mTableView->setContextMenu(tablePopup);
    }

    bodyLayout->addWidget(mTableView, 1);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("BOTTOM_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
        mPageTool = qobject_cast<TPageTool *>(uim()->getWidget("BOTTOM_TOOLBAR/PAGE_TOOL"));
        if (mPageTool != nullptr) {
            mPageTool->setPageSizeVisible(true);
            connect(mPageTool, SIGNAL(pageChanged(int,int)), this, SLOT(onPageChanged()));
        }
    }

    QWidget *detailWgt = new QWidget(this);
    QVBoxLayout *detailLayout = new QVBoxLayout(detailWgt);
    detailLayout->setSpacing(0);
    detailLayout->setMargin(0);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("DETAIL_TOOLBAR"))) {
        detailLayout->addWidget(toolbar, 0);
    }
    mDetailView = new TUiLoader(this);
    // 此ObjectName在其他逻辑中是有意义的，请勿删除
    mDetailView->setObjectName("__detail__");
    const QStringList typePlugins = config("uiloader_plugins").toStringList();
    if (typePlugins.size() > 0) {
        TopClassHelper::setUiLoaderPluginCallback(mDetailView,this,typePlugins);
    }
    connect(mDetailView, SIGNAL(dataChanged()), this, SLOT(onDetailDataChanged()));
    mDetailView->setProperty("SS_BG", "PANEL");
    mDetailView->setSelf(this);
    mDetailView->setScriptEngine(APP->scriptEngine());
    mDetailView->setUiStr(ui("detail").toString());
    detailLayout->addWidget(mDetailView);

    if (!ui("detail").isNull()) {
        mBodySplitter->addWidget(detailWgt);
        mBodySplitter->setStretchFactor(0, 1);
        mBodySplitter->setSizes(QList<int>{1, config("detail.perfect_width", 600).toInt()});
    } else {
        detailWgt->setVisible(false);
    }
}

/*!
 * \brief 初始化导航栏
 */
void TopTemplateClass4::initNaviView()
{
    mNaviType = str2NaviType(config("navi.__type__", "CategoryTreeView").toString());
    if (mNaviType == NaviType::None) {
        return;
    }

    TAccordion *accordion = new TAccordion(this);
    // 此ObjectName在其他逻辑中是有意义的，请勿删除
    accordion->setObjectName("__navi__");
    accordion->setProperty("SS_BG","NAVI");
    accordion->setMinimumWidth(TTHEME_DP(config("navi.min_width", 150).toInt()));
    accordion->setMaximumWidth(TTHEME_DP(config("navi.max_width", 300).toInt()));
    accordion->setAccordionLocation(TAccordion::AccordionLocation::Left);
    QVBoxLayout *naviLayout = new QVBoxLayout(accordion);
    naviLayout->setMargin(0);
    naviLayout->setSpacing(0);
    QLabel *naviLabel = new QLabel(tr("Navigation"));
    naviLabel->setProperty("SS_TYPE", "SUBHEADER");
    naviLayout->addWidget(naviLabel,0);
    naviLayout->addSpacing(TTHEME_DP(4));
    accordion->expandButton()->setFixedHeight(TTHEME_DP(40));
    accordion->expandButton()->setToolTip(tr("Show Navigation"));
    accordion->collapseButton()->setToolTip(tr("Hide Navigation"));
    accordion->setIsExpanded(config("navi.is_expanded", false).toBool());
    mMgtSplitter->insertWidget(0,accordion);

    if (mNaviType == NaviType::UiLoader) {
        mNaviView = new TUiLoader(this);
        TUiLoader *filterUi = qobject_cast<TUiLoader*>(mNaviView);
        filterUi->setSelf(this);
        filterUi->setScriptEngine(APP->scriptEngine());
        filterUi->setUiStr(ui("navi").toString());

        QFrame *naviWgt = new QFrame(this);
        QVBoxLayout *leftLayout = new QVBoxLayout(naviWgt);
        leftLayout->addWidget(mNaviView);
        QHBoxLayout *leftBtnsLayout = new QHBoxLayout();
        leftLayout->addLayout(leftBtnsLayout);
        QPushButton *resetBtn = new QPushButton(this);
        connect(resetBtn, SIGNAL(clicked(bool)), filterUi, SLOT(clearValues()));
        leftBtnsLayout->addWidget(resetBtn);
        resetBtn->setText(tr("Reset"));
        resetBtn->setIcon(TRES->icon("rotate-right"));
        QPushButton *filterBtn = new QPushButton(this);
        connect(filterBtn, SIGNAL(clicked(bool)), this, SLOT(refresh()));
        leftBtnsLayout->addWidget(filterBtn);
        filterBtn->setText(tr("Filter"));
        filterBtn->setIcon(TRES->icon("filter"));
        leftLayout->addStretch(1);
        naviLayout->addWidget(naviWgt);
    } else if (mNaviType == NaviType::AdvancedQuery) {
        Q_ASSERT_X(false, "TopTemplateClass4", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::NaviFilter) {
        Q_ASSERT_X(false, "TopTemplateClass4", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::CategoryTreeViewAndAdvancedQuery) {
        // tab1-CategoryTreeView
        TCategoryTreeView *categoryTreeView = new TCategoryTreeView(this);
        categoryTreeView->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviView->setVisible(false);
            return;
        }
        categoryTreeView->setItemList(rootChildList);
        categoryTreeView->setCheckable(config("navi.is_checkable").toBool());
        connect(categoryTreeView, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));

        // tab2-tab1-ToolBar
        QToolBar *toolbar = new QToolBar(this);
        toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        toolbar->setProperty("SS_SIZE", "SMALL");
        toolbar->setMovable(false);

        QAction *queryAction = toolbar->addAction(APP->getResIcon("search"), tr("Query"));
        connect(queryAction, SIGNAL(triggered()), this, SLOT(refresh()));
        toolbar->addSeparator();

        QAction* addAction = toolbar->addAction(APP->getResIcon("addto-circle-o"), tr("Add"));
        addAction->setToolTip(tr("Add query criteria"));
        connect(addAction, SIGNAL(triggered()), this, SLOT(onAddActionTriggered()));

        QAction* clearAction = toolbar->addAction(APP->getResIcon("eraser"), tr("Clear"));
        clearAction->setToolTip(tr("Clear all query criteria"));
        connect(clearAction, SIGNAL(triggered()), this, SLOT(onClearActionTriggered()));

        QAction* saveAsAction = toolbar->addAction(APP->getResIcon("save"), tr("Save As"));
        saveAsAction->setToolTip(tr("Save query criteria as file"));
        connect(saveAsAction, SIGNAL(triggered()), this, SLOT(onSaveAsActionTriggered()));

        QAction* openActtion = toolbar->addAction(APP->getResIcon("folder-open-o"), tr("Open"));
        openActtion->setToolTip(tr("Open query criteria from file"));
        connect(openActtion, SIGNAL(triggered()), this, SLOT(onOpenActtionTriggered()));

        QAction* saveAsCommonQueryAction = toolbar->addAction(APP->getResIcon("save-as"), tr("Save"));
        saveAsCommonQueryAction->setToolTip(tr("Save as a common query"));
        connect(saveAsCommonQueryAction, SIGNAL(triggered()), this, SLOT(onSaveAsCommonQueryActionTriggered()));

        // tab2-tab1-AdvancedQuery
        TAdvancedQuery *advancedQuery = new TAdvancedQuery(this);
        advancedQuery->setObjectName("AdvancedQuery");
        initAdvanceCfgMap();
        advancedQuery->setItemList(mAdvancedQueryCfgMap.value("advanced_items").toList());
        advancedQuery->setData(getDefaultQueryDataFromCfgMap());

        TScrollArea *scrollArea = new TScrollArea(this);
        scrollArea->setWidgetResizable(true);
        scrollArea->setWidget(advancedQuery);

        // tab2-tab1
        QMainWindow *mainWindow = new QMainWindow(this);
        mainWindow->addToolBar(toolbar);
        mainWindow->setCentralWidget(scrollArea);

        // tab2-tab2-TableView
        TTableView *commonQueryTable = new TTableView(this);
        commonQueryTable->setObjectName("CommonQueryTable");
        commonQueryTable->setShowGrid(false);
        commonQueryTable->setSelectionMode(QAbstractItemView::ExtendedSelection);
        commonQueryTable->verticalHeader()->setVisible(false);
        QVariantList headerItem;
        headerItem << QVariant() << TDataParse::variantList2Map(
                          QVariantList() <<"name" << "title" << "display" << tr("Query Name") << "resizeMode" << "Stretch" << "displayRole" << "$title");
        commonQueryTable->setHeaderItem(headerItem);
        commonQueryTable->setDataKeyList(QStringList() << "title" << "condition");
        commonQueryTable->setPrimaryKey("condition");
        connect(commonQueryTable, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(onCommonQueryTableDoubleClicked(const QModelIndex &)));

        QAction *deleleCommonQueryActtion = new QAction(APP->getResIcon("remove"), tr("Delete query"), this);
        connect(deleleCommonQueryActtion, SIGNAL(triggered()), this, SLOT(onDeleleCommonQueryActtionTriggered()));
        QMenu *contextMenu = new QMenu(this);
        contextMenu->addAction(deleleCommonQueryActtion);
        commonQueryTable->setContextMenu(contextMenu);

        // 刷新常用查询表格数据
        initCommonQueryMap();
        commonQueryTable->loadData(mAdvancedQueryTableDataMap.value("advance").toList());

        // 高级查询Tab
        TTabWidget *advancedQueryTabWidget = new TTabWidget(this);
        advancedQueryTabWidget->setObjectName("AdvancedQueryTabWidget");
        advancedQueryTabWidget->addTab(mainWindow, tr("Advanced Query"));
        advancedQueryTabWidget->addTab(commonQueryTable, tr("Common Query"));

        // 导航Tab
        mNaviView = new TTabWidget(this);
        TTabWidget *tabWidget = qobject_cast<TTabWidget *>(mNaviView);
        tabWidget->addTab(categoryTreeView, tr("Category"));
        tabWidget->addTab(advancedQueryTabWidget, tr("Advanced Query"));
        naviLayout->addWidget(mNaviView);
    } else {
        mNaviView = new TCategoryTreeView(this);
        mNaviView->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviView->setVisible(false);
            return;
        }
        TCategoryTreeView *categoryTreeView = qobject_cast<TCategoryTreeView *>(mNaviView);
        categoryTreeView->setItemList(rootChildList);
        categoryTreeView->setCheckable(config("navi.is_checkable").toBool());
        connect(categoryTreeView, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));
        naviLayout->addWidget(mNaviView);
    }
}

/**
* \brief 初始化高级查询配置数据
* mAdvancedQueryCfgMap结构如下：
* \code
    advance: {
      "advanced_items" : [
        {"name" : "attr_data.machine_code", "title" : "Machine Code" , "widgetType" : "LineEdit" , "wgt_prop" : "" , "optionList" : []},
        {"name" : "attr_data.machine_name", "title" : "Machine Name" , "widgetType" : "LineEdit" , "wgt_prop" : "" , "optionList" : []},
        {"name" : "title", "title" : "Title" , "widgetType" : "LineEdit" , "wgt_prop" : "" , "optionList" : []},
        {"name" : "class", "title" : "Class" , "widgetType" : "ComboBox" , "wgt_prop" : "" , "optionList" : "enum(tpm-machine-maintenance-plan-class)"}
      ],
      "file_suffix": ".conf"
      "condition" :{
        "condition": "1 and 2",
        "value": [
            {
                "ignoreEmpty": false,
                "name": "attr_data.machine_code",
                "operator": "LIKE",
                "title": "Machine Code",
                "valid": true,
                "value1": "00"
            },
            {
                "ignoreEmpty": false,
                "name": "attr_data.machine_name",
                "operator": "LIKE",
                "title": "Machine Name",
                "valid": true,
                "value1": "4"
            }
        ]
      }
    }
* \endcode
*/
void TopTemplateClass4::initAdvanceCfgMap()
{
    mAdvancedQueryCfgMap = config("advance").toMap();
    QVariantList queryItems = mAdvancedQueryCfgMap.value("advanced_items").toList();
    QVariantList newQueryItems;
    for (QVariant var : queryItems) {
        QVariantMap m = var.toMap();
        // 翻译title
        m.insert("title", ttr(m.value("title").toString()));
        // 转换枚举
        QVariantList optionList;
        QRegExp regExp("enum(.*)");
        if ((m.value("optionList").type() == QVariant::String) && (regExp.indexIn(m.value("optionList").toString()) != -1)) {
            if (regExp.capturedTexts().count() > 1) {
                optionList = TOPENM->enumList(regExp.capturedTexts().value(1).remove("(").remove(")"))->toComboList();
                m.insert("optionList", optionList);
            }
        }
        newQueryItems << m;
    }
    mAdvancedQueryCfgMap.insert("advanced_items", newQueryItems);
}

/**
* \brief 获取高级查询控件默认的查询条件
*/
QVariantMap TopTemplateClass4::getDefaultQueryDataFromCfgMap()
{
    QVariantMap conditionMap = config("advance").toMap().value("condition").toMap();
    QVariantList valueList = conditionMap.value("value").toList();
    for (QVariant &var: valueList) {
        QVariantMap m = var.toMap();
        m.insert("title", ttr(m.value("title").toString()));
        var = m;
    }
    conditionMap.insert("value", valueList);
    return conditionMap;
}

/**
* \brief 从高级查询配置中获取未使用条目的name和text，添加条目时使用
*/
QVariantList TopTemplateClass4::getAdvancedQueryFieldList()
{
    TAdvancedQuery *advancedQuery = findAdvancedQuery();
    if (advancedQuery == nullptr) {
        return QVariantList();
    }

    QVariantList selectedFieldDataLst = advancedQuery->getQueryData().value("value").toList();
    QStringList selectedFieldLst;
    for (QVariant val : selectedFieldDataLst){
        selectedFieldLst.append(val.toMap().value("name").toString());
    }

    QVariantList queryItems = mAdvancedQueryCfgMap.value("advanced_items").toList();
    QVariantList fieldList;
    for (QVariant var : queryItems) {
        QVariantMap m = var.toMap();
        QVariantMap itemMap;
        if(!selectedFieldLst.contains(m.value("name").toString())) {
            itemMap.insert("name", m.value("name").toString());
            itemMap.insert("title", ttr(m.value("title").toString()));
            fieldList << QVariant(itemMap);
        }
    }
    return fieldList;
}

/**
* \brief 从sys_conf表查找出已保存的常用查询数据
* 数据结构如下：
* \code
{
  "advance": [
    {
      "condition": {
        "condition": "1 and 2",
        "value": [
          {
            "ignoreEmpty": false,
            "name": "attr_data.machine_code",
            "operator": "LIKE",
            "title": "设备编号",
            "valid": true,
            "value1": "00"
          },
          {
            "ignoreEmpty": false,
            "name": "attr_data.machine_name",
            "operator": "LIKE",
            "title": "设备名称",
            "valid": true,
            "value1": "4"
          }
        ]
      },
      "title": "1"
    }
  ]
}
* \endcode
*/
void TopTemplateClass4::initCommonQueryMap()
{
    TSqlSelectorV2 selector;
    selector.setTable("sys_conf");
    selector.setField(QStringList() << "data");
    selector.setWhere("path", QStringList() << this->moduleName() << "AdvancedQuery", "@>");
    TError err;
    QVariant data = runSqlQueryOnThreadSync(TOPSQLTHREAD_SELECT_MAP,
                                            QVariant::fromValue(selector),
                                            &err);
    if (err.isValid()) {
        alertError(ttr("Load data failed!"), err.text());
        return;
    }
    mAdvancedQueryTableDataMap = TDataParse::jsonStr2Variant(data.toMap().value("data").toString()).toMap();
}

/**
* \brief 根据objname从导航栏中找到高级查询控件
*/
TAdvancedQuery *TopTemplateClass4::findAdvancedQuery()
{
    return mNaviView->findChild<TAdvancedQuery *>("AdvancedQuery");
}

/**
* \brief 根据objname从导航栏中找到常用查询tableView控件
*/
TTableView *TopTemplateClass4::findCommonQueryTable()
{
    return mNaviView->findChild<TTableView *>("CommonQueryTable");
}

/**
* \brief 根据objname从导航栏中找到高级查询TabWidget控件
*/
TTabWidget *TopTemplateClass4::findAdvancedQueryTabWidget()
{
    return mNaviView->findChild<TTabWidget *>("AdvancedQueryTabWidget");
}

/*!
 * \brief 解析导航栏配置
 */
QVariantList TopTemplateClass4::parseCategoryConfig()
{
    QVariantList categoryLst;

    QVariantList categoryConfig = config("navi.categories").toList();
    QString format = config("navi.format", "filter_by_enum").toString();
    if (format.compare("filter_by_enum") == 0) {
        for(const QVariant &item: categoryConfig) {
            QVariantMap itemMap = item.toMap();
            QString itemName         = itemMap.value("name").toString();
            QVariantList childrenLst = itemMap.value("children").toList();
            QString enumCfg          = itemMap.value("enum_children").toString();
            QStringList enumCheckedLst = itemMap.value("enum_checked").toStringList();
            QStringList enumInvisibleLst = itemMap.value("enum_invisible").toStringList();
            if (itemMap.contains("visible")) {
                itemMap.insert("VISIBLE", itemMap.take("visible"));
            }
            if (itemMap.contains("expand")) {
                itemMap.insert("EXPAND", itemMap.take("expand"));
            }
            TSqlWhereCompsiteV2 childWhere;
            childWhere.setLogic(TSqlWhereCompsiteV2::Logic_Or);
            QVariantList type = TOPENM->enumList(enumCfg)->toComboList();
            for (const QVariant &item: type) {
                QVariantMap enumMap = item.toMap();
                QString enumName = enumMap.value("name").toString();
                QVariantMap childMap;
                childMap.insert("name",enumName);
                childMap.insert("text",ttr(enumMap.value("text").toString()));
                childMap.insert("icon",enumMap.value("icon"));
                childMap.insert("data", itemName);
                if(enumCheckedLst.contains(enumName)) {
                    childMap.insert("checked",1);
                } else {
                    childMap.insert("checked",0);
                }
                if (enumInvisibleLst.contains(enumName)) {
                    childMap.insert("VISIBLE", 0);
                } else {
                    childMap.insert("VISIBLE",1);
                }
                childrenLst.append(childMap);
            }
            int checkedChildLen = 0;
            for (QVariant &item : childrenLst) {
                QVariantMap itemMap = item.toMap();
                QString name = itemMap.value("name").toString();
                if(itemMap.value("checked").toInt() == 1) {
                    childWhere.append(itemName, name);
                    checkedChildLen++;
                }
                item = itemMap;
            }
            if (checkedChildLen == childrenLst.length()) {
                itemMap.insert("checked", 1);
            } else {
                itemMap.insert("checked", 0);
            }
            mCategoryWhere.append(childWhere);

            itemMap.remove("children");
            itemMap.remove("enum_children");
            itemMap.remove("enum_checked");
            itemMap.remove("enum_invisible");
            itemMap.insert("text",ttr(itemMap.value("text").toString()));
            itemMap.insert("CHILDREN",childrenLst);

            categoryLst.append(itemMap);
        }
    } else if (format.compare("filter_by_sql_where") == 0) {
        return categoryConfig;
    }
    return categoryLst;
}

TopTemplateClass4::NaviType TopTemplateClass4::str2NaviType(const QString &iType)
{
    const QString type = iType.toUpper();
    if (type == "UILOADER") {
        return NaviType::UiLoader;
    } else if (type == "ADVANCEDQUERY") {
        return NaviType::AdvancedQuery;
    } else if (type == "NAVIFILTER") {
        return NaviType::NaviFilter;
    } else if (type == "CATEGORYTREEVIEWANDADVANCEDQUERY") {
        return NaviType::CategoryTreeViewAndAdvancedQuery;
    } else if (type == "NONE") {
        return NaviType::None;
    } else {
        return NaviType::CategoryTreeView;
    }
}

QString TopTemplateClass4::uiloaderWhere() const
{
    TUiLoader *filterUi = qobject_cast<TUiLoader*>(mNaviView);
    if (filterUi == nullptr) {
        return QString();
    }

    auto filterDataMap = filterUi->getAllValues().toVariant().toMap();

    QString searchSql;
    QMapIterator<QString, QVariant> iter(filterDataMap);
    while (iter.hasNext()) {
        iter.next();
        QString key = iter.key();
        QString value = iter.value().toString();
        if (value.isEmpty()) {
            continue;
        }
        auto obj = filterUi->getObject(key);
        if (obj != nullptr) {
            QVariantMap userData = obj->property("tui_user_data").toMap();
            if (userData.contains("field_name")) {
                key = userData.value("field_name").toString();
            }
            QString operatorStr = "=";
            if (userData.contains("operator")) {
                operatorStr = userData.value("operator").toString();
            }
            if (operatorStr.compare("like", Qt::CaseInsensitive) == 0) {
                value = QString("'%%1%'").arg(value);
            } else if (operatorStr.compare("in", Qt::CaseInsensitive) == 0) {
                value = QString("(%1)").arg(value);
            } else {
                if (iter.value().type() == QVariant::String) {
                    value = QString("'%1'").arg(value);
                }
            }
            QString searchStr = QString("%1 %2 %3").arg(key).arg(operatorStr).arg(value);
            if (!searchSql.isEmpty()) {
                searchSql.append(" and ");
            }
            searchSql.append(searchStr);
        }
    }
    return searchSql;
}
