﻿#ifndef TEMPLATECLASS5_H
#define TEMPLATECLASS5_H

#include <topcore/topclassabs.h>
#include <topcore/topclasshelper.h>

class TTableView;
class TSearchEntry;
class TPageTool;
class TTableView;
class TSplitter;

class TopTemplateClass5 : public TopClassAbs
{
    Q_OBJECT
public:
    explicit TopTemplateClass5(const QString &iModuleName = QString(""),
                               const QVariantMap &iUrlPars = QVariantMap(),
                               QWidget *iParent = nullptr);
    ~TopTemplateClass5();

public slots:
    void refreshMaster(bool iResetPage = true);
    void refreshSlave(bool iResetPage = true);
    void refreshDetail();
    TTableView *masterView() const;
    TTableView *slaveView() const;
    const QStringList masterQueryFields() const;
    const QStringList slaveQueryFields() const;
    QVariantList selectedMasterItems() const;
    QVariantList selectedMasterDataMaps() const;
    QVariantList selectedSlaveItems() const;
    QVariantList selectedSlaveDataMaps() const;
    void loadTreeData(const QVariantMap &iTreeData,
                      const QString &iSlavesKey = QString("data"));
    void loadTableData(const QVariantList &iTableData,
                       const QString &iSlavesKey = QString("data"));
    void loadMasterData(const QVariantList &iMasterData, int iRowCount = -1);
    void insertSlaveData(const QString &iKey, const QVariant &iValue);
    TUiLoader *uiLoader() const;
    void selectSlaveRow(const QVariant &iKeyVar);
    bool isDetailModified() const;
    void setIsDetailModified(bool iIsDetailModified);
signals:
    void isDetailModifiedChanged(bool iIsDetailModified);
private slots:
    void onMasterSelectionChanged();
    void onMasterPageChanged();
    void onSlaveSelectionChanged();
    void onSlavePageChanged();
    void onDetailChanged(bool iIsDetailModified);
private:
    void initUi();
    void resizeSplitterByRatio();
private:
    TSplitter *mBodySplitter = nullptr;
    TSplitter *mLeftSplitter = nullptr;
    TTableView *mMasterView = nullptr;
    TTableView *mSlaveView = nullptr;
    TSearchEntry *mMasterSearchEntry = nullptr;
    TPageTool *mMasterPageTool = nullptr;
    TSearchEntry *mSlaveSearchEntry = nullptr;
    TPageTool *mSlavePageTool = nullptr;
    TUiLoader *mUiLoader = nullptr;
    TopClassTableConf mMasterTableConf;
    TopClassTableConf mSlaveTableConf;
    QVariantList mSelectedMasterItems;
    QVariantList mSelectedSlaveItems;
    QVariantList mMasterData;
    QVariantMap mMasterSlaveDataMap;
    bool mIsDetailModified = false;
};

#endif // TEMPLATECLASS5_H
