﻿#include "toptemplateclass6.h"
#include <functional>
#include <QList>
#include <QWidget>
#include <QResizeEvent>
#include <QTabWidget>
#include <QToolBar>
#include <QVariantMap>
#include <QVBoxLayout>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#ifdef Q_OS_WIN
#include <tole/texcel.h>
#endif
#include <topcore/topcore.h>
#include <topcore/topclassabs.h>
#include <topcore/topclasshelper.h>
#include <topcore/topclasssqlthread.h>
#include <twidget/tfiledialog.h>
#include <twidget/tsplitter.h>
#include <twidget/ttreeview.h>
#include <twidget/tuidesigner.h>
#include <twidget/tuiloader.h>
#include <twidget/twidget.h>
#include <twidget/ttablechooserdialog.h>

/*!
 * \class TopTemplateClass6
 * \inmodule templateclass
 * \brief 树形报表模板，左右结构。
 */

TopTemplateClass6::TopTemplateClass6(const QString &iModuleNameStr, const QVariantMap &iUrlPars, QWidget *iParent)
    : TopClassAbs(iParent)
{
    initModule(iModuleNameStr, iUrlPars);
    QString licenseKey = config("sys_license_key").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("sys_lang_list").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    initUi();
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
    restoreSizeState();
    restoreObjectState(mBodySplitter);
    refreshActionState();
}

TopTemplateClass6::~TopTemplateClass6()
{
    saveSizeState();
    saveObjectState(mBodySplitter);
    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
}

TUiLoader *TopTemplateClass6::uiLoader()
{
    return mUiLoader;
}

/*!
 * \brief TopTemplateClass6::uiLoaderValues
 * \return 返回uiLoader的值
 */
QVariantMap TopTemplateClass6::uiLoaderValues()
{
    if (mUiLoader == nullptr) {
        return QVariantMap();
    }
    return mUiLoader->getAllValues().toVariant().toMap();
}

/*!
 * \brief TopTemplateClass6::onSetUiClicked
 * 编辑左侧ui界面
 */
void TopTemplateClass6::onSetUiClicked()
{
    QVariantMap langArgMap;
    QVariantMap langMap;
    langMap.insert("language_define",APP->languageDefine());
    langMap.insert("language",APP->language());
    langArgMap.insert("language_args",langMap);
    TUiDesigner *uiDesigner = new TUiDesigner(nullptr,langArgMap);
    uiDesigner->setScriptEngine(APP->scriptEngine());
    uiDesigner->setUiString(getUiStr());// 加载之前保存的Ui
    connect(uiDesigner, SIGNAL(dataApplied(QString)), this, SLOT(onUiTemplateApplied(QString)));
    uiDesigner->show();
}

/*!
 * \brief TopTemplateClass6::onUiTemplateApplied
 * \param iUiStr
 * 保存左侧ui到数据库
 */
void TopTemplateClass6::onUiTemplateApplied(const QString &iUiStr)
{
    QString tableName = config("condition_ui.table_name").toString();
    QString pathName = config("condition_ui.path_name").toString();
    QString pathValue = config("condition_ui.path_value").toString();
    QString field = config("condition_ui.field").toString();
    if (tableName.isEmpty() || pathName.isEmpty() || pathValue.isEmpty() || field.isEmpty()) {
        alertError(ttr("condition_ui config value is empty!"), "Error");
        return;
    }
    QVariantMap saveData;
    saveData.insert(field, QVariantMap{{"ui_str", iUiStr}});
    saveData.insert(pathName, pathValue);
    TSqlInserterV2 inserter;
    inserter.setTable(tableName);
    inserter.setField(QStringList() << field << pathName);
    inserter.setData(saveData);
    inserter.setUniqueField(pathName);
    inserter.setAutoIncrementField("id");
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_REPLACE_ROW, QVariant::fromValue(inserter));
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(dataRes.errText(), "Error");
        return;
    }
    mUiLoader->setUiStr(iUiStr);
}

/*!
 * \brief TopTemplateClass6::reopenTotalTab
 * \param iTotalData
 * 关闭tab所有页，重新打开新的总表
 */
void TopTemplateClass6::reopenTotalTab(const QVariantMap &iTotalData)
{
    try {
        //关闭所有tab页
        mTabWidget->clear();
        clearTabLst();
        //新建总表tab页
        TTreeView *totalTree = getSubTab(iTotalData);
        if (QMenu *menu = qobject_cast<QMenu *>(uim()->getWidget("TREEVIEW_POPUP"))) {
            totalTree->setContextMenu(menu);
        }
        totalTree->setSelectionBehavior(QAbstractItemView::SelectItems);
        totalTree->setSelectionMode(QAbstractItemView::ExtendedSelection);
        connect(totalTree, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(onTreeViewItemDoubleClicked(QModelIndex)));
        mTabWidget->addTab(totalTree, iTotalData["name"].toString());
    } catch (const TError &e) {
        alertError(ttr("ERROR"), e.text());
    } catch (...) {
        alertError(ttr("ERROR"), ttr("Unkown Error!"));
    }
}

/*!
 * \brief TopTemplateClass6::getSubTab
 * \param iSubData
 * {
    "name":"test",
    "view":{
        "data_keys":[

        ],
        "horizontal_header":[
            {
                "name":"zone",
                "display":"Zone",
                "displayRole":"$zone"
            }
        ]
    },
    "data":[

    ]
}
 * \return 根据iSubData配置，创建新的树形控件
 */
TTreeView *TopTemplateClass6::getSubTab(const QVariantMap &iSubData)
{
    TTreeView *treeView = new TTreeView(this);
    TopClassTableConf treeConf;
    if (!iSubData.contains("view")) {
        throw TError(ttr("view doesn't exist!"), "ERROR");
    }
    if (iSubData.value("view").type() != QVariant::Map) {
        throw TError(ttr("view type != map"), "ERROR");
    }
    TopClassHelper::parseTableConf0(this, iSubData["view"].toMap(), &treeConf);
    treeView->setHeaderItem(treeConf.horizontalHeaders);
    treeView->setDataKeyList(treeConf.dataKeys);
    QVariantList treeData = iSubData["data"].toList();
    TopClassHelper::formatTableData(this, &treeConf, treeData);
    treeView->loadTreeData(treeData);
    treeView->collapseAll();
    return treeView;
}

/*!
 * \brief TopTemplateClass6::expandAll
 * 展开当前页的树形控件
 */
void TopTemplateClass6::expandAll()
{
    if (TTreeView *treeview = qobject_cast<TTreeView *>(mTabWidget->currentWidget())) {
        treeview->expandAll();
    }
}

/*!
 * \brief TopTemplateClass6::collapseAll
 * 折叠当前页的树形控件
 */
void TopTemplateClass6::collapseAll()
{
    if (TTreeView *treeview = qobject_cast<TTreeView *>(mTabWidget->currentWidget())) {
        treeview->collapseAll();
    }
}

/*!
 * \brief TopTemplateClass6::exportExcel
 * 列出所有tab页，选择需要的tab页，导出到一个excel文件
 */
void TopTemplateClass6::exportExcel()
{
#ifdef Q_OS_WIN
    TTableChooserDialog *chooseDlg = new TTableChooserDialog(this);
    chooseDlg->setTitle(ttr("Select Export Tab"));
    QVariantList headerItems;
    headerItems << QVariant();
    headerItems << TDataParse::stringList2VariantMap(
                       QStringList() << "name" << "name" << "display" << ttr("Name")
                       << "displayRole" << "$name"<<"resizeMode" << "Stretch");
    chooseDlg->setHeaderItem(headerItems);
    chooseDlg->setPrimaryKey("index");
    chooseDlg->setDataKeyList(QStringList() << "index" << "name");
    chooseDlg->loadAllData(allTabInfo());
    QVariantList selectedTabData = chooseDlg->run();
    if (selectedTabData.length() == 0) {
        return;
    }

    QString fileName = QString("%1 %2.xlsx").arg(moduleName().replace("-", " ")).arg(QDateTime::currentDateTime().toString("yyyyMMddhhmmss"));
    TFileDialog *fileDlg = new TFileDialog(ttr("Please select save file"), fileName);
    fileDlg->setAcceptMode("AcceptSave");
    QStringList strLst = fileDlg->run();
    if (strLst.length() == 0 ) {
        return;
    }
    QString absoluteFileName = strLst.first();

    TExcel *excel = new TExcel(this);
    QAxObject *obj = excel->addBook();
    if (obj == nullptr) {
        excel->quitExcel();
        alertError(ttr("Export failed!"));
        return;
    }

    std::function<QVariantList(const QVariantList, const QString)> transformTreeData;
    transformTreeData = [&transformTreeData](const QVariantList &iDataLst, const QString &iChildrenKey){
        QVariantList allDataLst;
        for (QVariant item : iDataLst) {
            QVariantMap itemMap = item.toMap();
            QVariantList childDataLst = itemMap.take(iChildrenKey).toList();
            allDataLst.append(itemMap);
            if (childDataLst.length() > 0) {
                 allDataLst.append(transformTreeData(childDataLst, iChildrenKey));
            }
        }
        return allDataLst;
    };

    std::function<void(TTreeView*)> exportToSheet = [&excel, &transformTreeData](TTreeView* iTreeView){
        if (iTreeView == nullptr) {
            return;
        }
        QVariantList allDataLst = iTreeView->allDataMap();
        allDataLst = transformTreeData(allDataLst, "CHILDREN");
        QVariantList headerItems = iTreeView->headerItem();
        if (allDataLst.length() == 0) {
            return;
        }

        int rowInt = 1;
        // 表头字段
        for (int i = 0; i < headerItems.length(); i++) {
            QVariantMap itemMap = headerItems[i].toMap();
            QString colName = itemMap.value("display").toString();
            QString colRange = QString("%1%2").arg(QChar('A' + i)).arg(QString::number(rowInt));
            QVariantMap font_map = QVariantMap({{"Size", 12},
                                                {"Bold", true},
                                                {"Color", QColor("black")}});
            excel->setRangeValue(colRange, colName);
            excel->setRangeFont(colRange, font_map);
            excel->autoFitColumns(QChar('A' + i));
            excel->setRangeHorizontalAlignment(colRange, -4108);
        }
        rowInt++;

        // 写值
        for (QVariant dataVar: allDataLst) {
            QVariantMap dataVarMap = dataVar.toMap();
            int colInt = 0;
            for (QVariant item: headerItems) {
                QVariantMap itemMap = item.toMap();
                QString keyStr = itemMap.value("displayRole").toString().mid(1);
                QString cellRange = QString("%1%2").arg(QChar('A' + colInt)).arg(QString::number(rowInt));
                excel->setRangeValue(cellRange, dataVarMap.value(keyStr));
                excel->setRangeHorizontalAlignment(cellRange, -4108);
                excel->autoFitColumns(QChar('A' + colInt));
                colInt++;
            }
            rowInt++;
        }

        // 边框
        int row = excel->getUsedRows();
        int column = excel->getUsedColumns();
        QString rangecolunmn = QString(65 + column - 1);
        QString used_range = QString("A1:%1%2").arg(rangecolunmn).arg(row);
        QVariantMap border = QVariantMap({{"Color", -4165632},
                                          {"LineStyle", 1},
                                          {"TintAndShader", 0},
                                          {"Weight", 2}}) ;
        excel->setRangeBorder(used_range, border);
    };

    excel->setExcelDisplayAlerts(true);
    excel->setExcelVisible(true);
    for (int i = 0; i < selectedTabData.length(); i++) {
        QVariantMap itemMap = selectedTabData[i].toMap();
        int index = itemMap.value("index").toInt();
        QString tabName = itemMap.value("name").toString();
        if ((i + 1) > excel->getSheetsCount()) {
            excel->appendSheet(tabName);
        }
        excel->selectSheet(i + 1);
        excel->setSheetName(tabName);

        TTreeView *treeView = qobject_cast<TTreeView *>(mTabWidget->widget(index));
        exportToSheet(treeView);
    }

    excel->saveAsBook(absoluteFileName.replace("/", "\\"));
    excel->closeBook();
    excel->quitExcel();
    alertOk(ttr("Export success!"));
#endif
}

QVariantList TopTemplateClass6::allTabInfo()
{
    QVariantList tabDataLst;
    for (int i = 0; i < mTabWidget->count(); i++) {
        QVariantMap tabInfo;
        tabInfo.insert("index", i);
        tabInfo.insert("name", mTabWidget->tabText(i));
        tabDataLst.append(tabInfo);
    }
    return tabDataLst;
}

TTreeView *TopTemplateClass6::getTreeView(int iIndex)
{
    return qobject_cast<TTreeView *>(mTabWidget->widget(iIndex));
}

QVariantList TopTemplateClass6::getTreeDataLst(int iIndex)
{
    TTreeView *treeView = qobject_cast<TTreeView *>(mTabWidget->widget(iIndex));
    if (treeView) {
        return treeView->allDataMap();
    }
    return QVariantList();
}

/*!
 * \brief TopTemplateClass6::onTreeViewItemDoubleClicked
 * \param iIndex
 * 双击总表的单元格后打开子表
 */
void TopTemplateClass6::onTreeViewItemDoubleClicked(const QModelIndex &iIndex)
{
    if (mSubTabMap.contains(iIndex)) {
        mTabWidget->setCurrentWidget(mSubTabMap.value(iIndex));
    } else {
        TTreeView *totalTree = qobject_cast<TTreeView *>(mTabWidget->widget(0));
        if (totalTree->currentRowDataMap().isNull()) {
            return;
        }
        QVariantMap cellData;
        cellData.insert("row", totalTree->currentRowDataMap().toMap());
        cellData.insert("column", totalTree->headerItem().at(iIndex.column()).toMap().value("name"));
        QVariantMap subData = callHooksQuick("getSubData", QVariantList() << cellData).toVariant().toMap();
        TTreeView *subTree = getSubTab(subData);
        subTree->setSelectionBehavior(QAbstractItemView::SelectRows);
        subTree->setSelectionMode(QAbstractItemView::ExtendedSelection);
        if (QMenu *menu = qobject_cast<QMenu *>(uim()->getWidget("TREEVIEW_POPUP"))) {
            subTree->setContextMenu(menu);
        }
        mTabWidget->addTab(subTree, subData["name"].toString());
        mTabWidget->setCurrentWidget(subTree);
        mSubTabMap.insert(iIndex, subTree);
    }
}

/*!
 * \brief TopTemplateClass6::onTabClosedButtomClicked
 * \param index
 * 关闭tab页
 */
void TopTemplateClass6::onTabClosedButtomClicked(int index)
{
    TTreeView *closingView = qobject_cast<TTreeView *>(mTabWidget->widget(index));
    if(closingView->close()) {
        mTabWidget->removeTab(index);
        QMap<QModelIndex, TTreeView*>::iterator iter;
        for (iter = mSubTabMap.begin(); iter != mSubTabMap.end();) {
            TTreeView *treeView = iter.value();
            if (treeView == closingView) {
                delete treeView;
                treeView = nullptr;
                mSubTabMap.erase(iter++);
            } else {
                iter++;
            }
        }
    }
}

/*!
 * \brief TopTemplateClass6::initUi
 * 初始化界面
 */
void TopTemplateClass6::initUi()
{
    mBodySplitter = new TSplitter(this);
    mBodySplitter->setObjectName("mBodySplitter");
    setCentralWidget(mBodySplitter);
    //left
    TWidget *leftWidget = new TWidget(mBodySplitter);
    QVBoxLayout *leftLayout = new QVBoxLayout(leftWidget);
    leftLayout->setMargin(0);
    leftLayout->setSpacing(0);
    mBodySplitter->addWidget(leftWidget);
    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("CONDITION_TOOLBAR"))) {
        leftLayout->addWidget(toolbar, 0);
    }
    mUiLoader = new TUiLoader(this);
    const QStringList typePlugins = config("uiloader_plugins").toStringList();
    if (typePlugins.size() > 0) {
        TopClassHelper::setUiLoaderPluginCallback(mUiLoader,this,typePlugins);
    }
    mUiLoader->setScriptEngine(APP->scriptEngine());
    mUiLoader->setSelf(this);
    mUiLoader->setUiStr(getUiStr());
    leftLayout->addWidget(mUiLoader, 1);
    // right
    TWidget *rightWidget = new TWidget(mBodySplitter);
    QVBoxLayout *rightLayout = new QVBoxLayout(rightWidget);
    rightLayout->setMargin(0);
    rightLayout->setSpacing(0);
    mBodySplitter->addWidget(rightWidget);
    mTabWidget = new QTabWidget(rightWidget);
    mTabWidget->setStyleSheet("QTabBar {qproperty-tabsClosable: 1;}");
    mTabWidget->setTabsClosable(true);
    connect(mTabWidget, SIGNAL(tabCloseRequested(int)), this, SLOT(onTabClosedButtomClicked(int)));
    if (QToolBar *toolbar = qobject_cast<QToolBar *>(uim()->getWidget("MAIN_TOOLBAR"))) {
        rightLayout->addWidget(toolbar, 0);
    }
    rightLayout->addWidget(mTabWidget);

    int width = this->width();
    int tableWidth = width * config("width_ratio").toDouble();
    mBodySplitter->setSizes(QList<int>{tableWidth, width - tableWidth});
}

/*!
 * \brief TopTemplateClass6::getUiStr
 * \return 根据module中左侧ui配置，从表中查找左侧ui的界面数据
 */
QString TopTemplateClass6::getUiStr()
{
    QString tableName = config("condition_ui.table_name").toString();
    QString pathName = config("condition_ui.path_name").toString();
    QString pathValue = config("condition_ui.path_value").toString();
    QString field = config("condition_ui.field").toString();
    QVariantMap fieldFormat = config("condition_ui.field_format").toMap();
    if (tableName.isEmpty() || pathName.isEmpty() || pathValue.isEmpty() || field.isEmpty()) {
        return QString();
    }
    TSqlSelectorV2 selector;
    selector.setTable(tableName);
    if (!field.isEmpty()) {
        selector.setField(field);
    }
    selector.setFieldFormat(fieldFormat);
    selector.setWhere(pathName, pathValue);
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_VALUE, QVariant::fromValue(selector));
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(dataRes.errText(), "Error");
        return QString();
    }
    return dataRes.data().toMap().value("ui_str").toString();
}

/*!
 * \brief TopTemplateClass6::clearTabLst
 * 清理所有打开的tab页控件
 */
void TopTemplateClass6::clearTabLst()
{
    QMap<QModelIndex, TTreeView*>::iterator iter;
    for (iter = mSubTabMap.begin(); iter != mSubTabMap.end();) {
        TTreeView* treeView = iter.value();
        delete treeView;
        treeView = nullptr;
        mSubTabMap.erase(iter++);
    }
}
