﻿#include "toptemplateclass7.h"
#include <QHBoxLayout>
#include <QLabel>
#include <QMenu>
#include <QPushButton>
#include <QTimer>
#include <QToolBar>
#include <QVBoxLayout>
#include <QWidget>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/ttheme.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <topcore/topclasshelper.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topcore.h>
#include <twidget/taccordion.h>
#include <twidget/tpagetool.h>
#include <twidget/tsplitter.h>
#include <twidget/ttableview.h>
#include <twidget/tuiloader.h>
#include <twidget/taccordion.h>

/*!
 * \class TopTemplateClass7
 * \inmodule templateclass
 * \brief 带筛选条件，详细信息的报表展示。支持搜索，展示详情
 */

TopTemplateClass7::TopTemplateClass7(const QString &iModuleNameStr,
                                     const QVariantMap &iUrlPars,
                                     QWidget *iParent)
    : TopClassAbs(iParent)
{
    initModule(iModuleNameStr, iUrlPars);
    QString licenseKey = config("sys_license_key").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("sys_lang_list").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    TopClassHelper::parseTableConf0(this, "view", &mTableConf);
    initUi();
    connect(this, SIGNAL(viewConfChanged()),
            this, SLOT(onViewConfChanged()));
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
    restoreSizeState();
    restoreObjectState(mBodySplitter);
    restoreObjectState(mTableView);
    refreshActionState();
    QTimer::singleShot(0, this, SLOT(refresh()));
}

TopTemplateClass7::~TopTemplateClass7()
{
    saveSizeState();
    saveObjectState(mBodySplitter);
    saveObjectState(mTableView);
    if (isHookExists("onDestroy")) {
        callHooks("onDestroy");
    }
}

QString TopTemplateClass7::detailUid() const
{
    return mDetailUid;
}

void TopTemplateClass7::setDetailUid(const QString &iUid, bool iForceRefresh)
{
    if (iForceRefresh || mDetailUid != iUid) {
        mDetailUid = iUid;
        emit detailUidChanged(iUid);
    }
}

QVariantList TopTemplateClass7::selectedItems() const
{
    return mSelectedItems;
}

QVariantList TopTemplateClass7::selectedDataMaps() const
{
    return mTableView->selectedRowDataMaps(true);
}

TTableView *TopTemplateClass7::tableView()
{
    return mTableView;
}

/*!
 * \brief TopTemplateClass7::refresh 刷新数据
 * \param iResetPage 表示是否要重置分页
 */
void TopTemplateClass7::refresh(bool iResetPage)
{
    TSqlSelectorV2 selector;
    if (!mTableConf.dbSql.isEmpty()) {
        selector.setTable(QString("(%1) _TEMP_TABLE_").arg(mTableConf.dbSql));
    } else if (!mTableConf.dbTableName.isEmpty()) {
        selector.setTable(mTableConf.dbTableName);
        // 搜索栏
        QString searchSql = searchWhere(searchUiValues());
        if (!searchSql.isEmpty()) {
            selector.addWhere(searchSql);
        }
    } else {
        //db_sql为空和db_table_name为空时不作处理
        return;
    }
    selector.setField(mTableConf.queryFields);
    selector.setFieldFormat(mTableConf.fieldFormatMap);
    selector.setReturnRowCount(true);
    // 分页
    TopClassHelper::handleSearchPageOnQuery(nullptr, mPageTool, iResetPage, &selector);
    // 排序
    TopClassHelper::handleOrderOnQuery(mTableView, &mTableConf, &selector);
    // 只获取有效数据
    if (!mTableConf.dbDelFlagKey.isEmpty()) {
        selector.addWhere(QString("%1 = 0 OR %1 IS NULL").arg(mTableConf.dbDelFlagKey));
    }
    // 过滤项
    if(!mTableConf.dbFilter.isEmpty()) {
        selector.addWhere(mTableConf.dbFilter);
    }

    loading(ttr("Loading"));
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
    unloading();
    TDataResponse response(data.toMap());
    if (response.hasError()) {
        alertError(ttr("Load data failed!"));
    } else {
        if (mPageTool != nullptr) {
            mPageTool->setRowCount(response.dataCount(), true);
        }
        QVariantList tableData = response.data().toList();
        TopClassHelper::formatTableData(this, &mTableConf, tableData);
        if (isHookExists("handleTableData")) {
            tableData = callHooksQuick("handleTableData", QVariantList() << QVariant(tableData)).toVariant().toList();
        }
        mTableView->loadData(tableData);
        mSelectedItems.clear();
        setDetailUid(QString());
        alertOk(ttr("Data loaded"));
    }
}

/*!
 * \brief TopTemplateClass7::refreshDetail 刷新详情数据
 * \param iUid 详情id
 */
void TopTemplateClass7::refreshDetail(const QString &iUid)
{
    if (iUid.isEmpty()) {
        mDetailUiLoader->loadValues(QVariantMap(), true);
        return;
    }

    TSqlSelectorV2 selector;
    if (!mTableConf.dbSql.isEmpty()) {
        selector.setTable(QString("(%1) _TEMP_TABLE_").arg(mTableConf.dbSql));
    } else {
        selector.setTable(mTableConf.dbTableName);
    }
    selector.setWhere(mTableConf.primaryKey, iUid);
    selector.setField(mTableConf.queryFields);
    selector.setFieldFormat(mTableConf.fieldFormatMap);
    TError err;
    QVariant data = runSqlQueryOnThreadSync(TOPSQLTHREAD_SELECT_MAP,
                                            QVariant::fromValue(selector),
                                            &err);
    if (err.isValid()) {
        alertError(ttr("Load data failed!"), err.text());
        return;
    }
    QVariantMap dataMap = data.toMap();
    mDetailUiLoader->loadValues(dataMap, true);
}

/*!
 * \brief TopTemplateClass7::searchUiLoader
 * \return 搜索ui控件
 */
TUiLoader *TopTemplateClass7::searchUiLoader()
{
    return mSearchUiLoader;
}

/*!
 * \brief TopTemplateClass7::searchUiValues
 * \return 搜索ui的数据
 */
QVariantMap TopTemplateClass7::searchUiValues()
{
    return mSearchUiLoader->getAllValues().toVariant().toMap();
}

/*!
 * \brief TopTemplateClass7::resetSearchUiLoader 清除搜索ui的数据
 */
void TopTemplateClass7::resetSearchUiLoader()
{
    mSearchUiLoader->clearValues();
}

/*!
 * \brief TopTemplateClass7::detailUiLoader
 * \return 详情ui控件
 */
TUiLoader *TopTemplateClass7::detailUiLoader()
{
    return mDetailUiLoader;
}

/*!
 * \brief TopTemplateClass7::searchWhere
 * \param iSearchMap
 * \return 根据iSearchMap组装搜索sql语句
 */
QString TopTemplateClass7::searchWhere(const QVariantMap &iSearchMap)
{
    QString searchSql;
    QMapIterator<QString, QVariant> iter(iSearchMap);
    while (iter.hasNext()) {
        iter.next();
        QString key = iter.key();
        QString value = iter.value().toString();
        if (value.isEmpty()) {
            continue;
        }
        auto obj = mSearchUiLoader->getObject(key);
        if (obj != nullptr) {
            QVariantMap userData = obj->property("tui_user_data").toMap();
            if (userData.contains("field_name")) {
                key = userData.value("field_name").toString();
            }
            QString operatorStr = "=";
            if (userData.contains("operator")) {
                operatorStr = userData.value("operator").toString();
            }
            if (operatorStr.compare("like", Qt::CaseInsensitive) == 0) {
                value = QString("'%%1%'").arg(value);
            } else if (operatorStr.compare("in", Qt::CaseInsensitive) == 0) {
                value = QString("(%1)").arg(value);
            } else {
                if (iter.value().type() == QVariant::String) {
                    value = QString("'%1'").arg(value);
                }
            }
            QString searchStr = QString("%1 %2 %3").arg(key).arg(operatorStr).arg(value);
            if (!searchSql.isEmpty()) {
                searchSql.append(" and ");
            }
            searchSql.append(searchStr);
        }
    }
    return searchSql;
}

void TopTemplateClass7::selectRow(const QString &iId)
{
    mTableView->selectRow(iId);
}

/*!
 * \brief TopTemplateClass7::onSelectionChanged 选中一条表格数据，详情id改变
 */
void TopTemplateClass7::onSelectionChanged()
{
    mSelectedItems = mTableView->selectedPrimaryKeys();
    QString curId;
    if (!mSelectedItems.isEmpty()) {
        curId = mSelectedItems.first().toString();
    }
    if (mDetailUid != curId) {
        setDetailUid(curId);
    }
    refreshActionState();
}

/*!
 * \brief TopTemplateClass7::onPageChanged 响应翻页控件的页面改变
 */
void TopTemplateClass7::onPageChanged()
{
    refresh(false);
}

void TopTemplateClass7::onViewConfChanged()
{
    TopClassHelper::parseTableConf0(this, "view", &mTableConf);
}

/*!
 * \brief TopTemplateClass7::initUi 初始化界面
 */
void TopTemplateClass7::initUi()
{
    mBodySplitter = new TSplitter(this);
    mBodySplitter->setObjectName("mBodySplitter");
    setCentralWidget(mBodySplitter);
    initSearchUiLoader();
    initTableView();
    initDetailUiLoader();

    QString viewRatio = config("view_ratio").toString();
    QRegularExpression regExp("(?<search>\\d+):(?<tableview>\\d+):(?<detail>\\d+)");
    QRegularExpressionMatch match = regExp.match(viewRatio);
    if (match.hasMatch()) {
        double detailRatio =  match.captured("search").toDouble() /
                (match.captured("search").toDouble() + match.captured("tableview").toDouble() + match.captured("detail").toDouble());
        double tableviewRation =  match.captured("tableview").toDouble() /
                (match.captured("search").toDouble() + match.captured("tableview").toDouble() + match.captured("detail").toDouble());
        int width = this->width();
        int searchWidth = static_cast<int>(width * detailRatio);
        int tableviewWidth = static_cast<int>(width * tableviewRation);
        mBodySplitter->setSizes(QList<int>{ searchWidth, tableviewWidth , width-searchWidth-tableviewWidth});
    }
}

void TopTemplateClass7::initSearchUiLoader()
{
    mSearchUiLoader = new TUiLoader(this);
    mSearchUiLoader->setProperty("SS_BG", "PANEL");
    mSearchUiLoader->setSelf(this);
    mSearchUiLoader->setScriptEngine(APP->scriptEngine());
    mSearchUiLoader->setUiStr(ui("search").toString());

    QWidget *searchWidget = new QWidget(this);
    searchWidget->setObjectName("__navi__");
    searchWidget->setMinimumWidth(TTHEME_DP(config("search.min_width", 150).toInt()));
    searchWidget->setMaximumWidth(TTHEME_DP(config("search.max_width", 300).toInt()));
    QVBoxLayout *searchLayout = new QVBoxLayout(searchWidget);
    searchLayout->setMargin(0);
    searchLayout->setSpacing(0);
    QLabel *naviLabel = new QLabel(ttr("Navigation"));
    naviLabel->setProperty("SS_TYPE", "SUBHEADER");
    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("SEARCH_TOOLBAR"))) {
        searchLayout->addWidget(toolbar, 0);
        searchLayout->addSpacing(TTHEME_DP(4));
    }
    searchLayout->addWidget(mSearchUiLoader, 1);
    searchLayout->addStretch(1);

    if (config("search_ui_set.is_navi", 1).toBool()) {
        TAccordion *accordion = new TAccordion(this);
        accordion->setProperty("SS_BG","NAVI");
        accordion->setMinimumWidth(TTHEME_DP(config("navi.min_width", 150).toInt()));
        accordion->setMaximumWidth(TTHEME_DP(config("navi.max_width", 300).toInt()));
        accordion->setAccordionLocation(TAccordion::AccordionLocation::Left);
        QVBoxLayout *naviLayout = new QVBoxLayout(accordion);
        naviLayout->setMargin(0);
        naviLayout->setSpacing(0);
        naviLayout->addWidget(naviLabel, 0);
        naviLayout->addSpacing(TTHEME_DP(4));
        naviLayout->addWidget(searchWidget);
        accordion->expandButton()->setFixedHeight(TTHEME_DP(40));
        accordion->expandButton()->setToolTip(ttr("Show Navigation"));
        accordion->collapseButton()->setToolTip(ttr("Hide Navigation"));
        accordion->setIsExpanded(config("search_ui_set.navi_is_expand").toBool());
        mBodySplitter->insertWidget(0,accordion);
    } else {
        searchLayout->insertWidget(0, naviLabel);
        searchLayout->addSpacing(TTHEME_DP(4));
        mBodySplitter->insertWidget(0, searchWidget);
    }
}

/*!
 * \brief TopTemplateClass7::initDetailUiLoader 初始化详情ui
 */
void TopTemplateClass7::initDetailUiLoader()
{
    if (!ui("detail").toString().isEmpty()) {
        QWidget *detailWidget = new QWidget(this);
        QVBoxLayout *detailLayout = new QVBoxLayout(detailWidget);
        detailLayout->setSpacing(0);
        detailLayout->setMargin(0);

        if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("DETAIL_TOOLBAR"))) {
            detailLayout->addWidget(toolbar, 0);
        }
        mDetailUiLoader = new TUiLoader(this);
        mDetailUiLoader->setProperty("SS_BG", "PANEL");
        mDetailUiLoader->setSelf(this);
        mDetailUiLoader->setScriptEngine(APP->scriptEngine());
        mDetailUiLoader->setUiStr(ui("detail").toString());
        detailLayout->addWidget(mDetailUiLoader);
        mBodySplitter->addWidget(detailWidget);
        connect(this, SIGNAL(detailUidChanged(QString)),
                this, SLOT(refreshDetail(QString)));
    }
}

/*!
 * \brief TopTemplateClass7::initTableView
 */
void TopTemplateClass7::initTableView()
{
    QWidget *centerWidget = new QWidget(this);
    QVBoxLayout *centerLayout = new QVBoxLayout(centerWidget);
    centerLayout->setMargin(0);
    centerLayout->setSpacing(0);
    mTableView = new TTableView(this);
    mTableView->setObjectName("mTableView");
    mTableView->setDataKeyList(mTableConf.dataKeys);
    mTableView->setPrimaryKey(mTableConf.primaryKey);
    QVariantList headerItems = mTableConf.horizontalHeaders;
    headerItems.prepend(QVariant());
    mTableView->setHeaderItem(headerItems);

    connect(mTableView->horizontalHeader(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)),
            this, SLOT(refresh()));
    connect(mTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onSelectionChanged()));
    if (QToolBar *toolbar = qobject_cast<QToolBar *>(uim()->getWidget("MAIN_TOOLBAR"))) {
        centerLayout->addWidget(toolbar, 0);
    }
    centerLayout->addWidget(mTableView);
    if (QMenu *menu = qobject_cast<QMenu *>(uim()->getWidget("TABLEVIEW_POPUP"))) {
        mTableView->setContextMenu(menu);
    }

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("BOTTOM_TOOLBAR"))) {
        centerLayout->addWidget(toolbar, 0);
        mPageTool = qobject_cast<TPageTool *>(uim()->getWidget("BOTTOM_TOOLBAR/PAGE_TOOL"));
        if (mPageTool != nullptr) {
            mPageTool->setPageSizeVisible(true);
            connect(mPageTool, SIGNAL(pageChanged(int,int)), this, SLOT(onPageChanged()));
        }
    }
    mBodySplitter->addWidget(centerWidget);
}
